package Class::DBI::Plugin::SerializeColumns;

use strict;
use warnings;
use Carp;
use UNIVERSAL::require;
use vars qw($VERSION);
$VERSION='0.01';

sub import {
	my $class = shift;
	my $pkg   = caller(0);
	
	$pkg->mk_classdata('columns_serializer'=>'Storable');
	
	no strict 'refs';
	*{"$pkg\::serialize_columns"} = sub {
		my($class, $parent_column, @colnames) = @_;
		$class->columns(TEMP=>@colnames);
		
		my $before_create_update=sub{
			my $self = shift;
			my %hash;
			for my $s_column (@colnames) {	
				$hash{$s_column}=$self->get($s_column);
			}
			my $ref=$self->_serialize_columns_freeze(\%hash);
			$self->set($parent_column,$ref);
		};
		
		$pkg->add_trigger('before_create',$before_create_update);
		$pkg->add_trigger('before_update',$before_create_update);
		
		$class->add_trigger('select',sub{
			my $self = shift;
			my $property = $self->_serialize_columns_thaw($self->get($parent_column)) || {};
			for my $s_column (keys %$property) {
				$self->set($s_column,$property->{$s_column});
			}
		});
		
	};
	
	*{"$pkg\::_columns_serializer"} = \&_columns_serializer;
	*{"$pkg\::_serialize_columns_freeze"} = \&_serialize_columns_freeze;
	*{"$pkg\::_serialize_columns_thaw"} =  \&_serialize_columns_thaw;
}

sub _columns_serializer{
	my $self = shift;
	my $serializer = "Class::DBI::Plugin::SerializeColumns::" . $self->columns_serializer;
	$serializer->require;
	return $self->_croak(qq(couldn't load serializer "$serializer" : ) . $@) if $@;
	return $serializer;
}

sub _serialize_columns_freeze{
	my($self, $var) = @_;
	my $serializer = $self->_columns_serializer;
	my $freeze;
	eval{
		$freeze=$serializer->freeze($var);
	};
	return $self->_croak(qq(couldn't freeze data : ) . $@) if $@;
	return $freeze;
}

sub _serialize_columns_thaw{
	my($self, $freeze) = @_;
	my $serializer = $self->_columns_serializer;
	my $thaw;
	eval{
		$thaw=$serializer->thaw($freeze);
	};
	return $self->_croak(qq(couldn't thaw data : ) . $@) if $@;
	return $thaw;
}

1;

__END__
# Below is stub documentation for your module. You'd better edit it!

=head1 NAME

Class::DBI::Plugin::SerializeColumns - Perl extension for blah blah blah

=head1 SYNOPSIS

  use Class::DBI::Plugin::SerializeColumns;
  blah blah blah

=head1 DESCRIPTION

Stub documentation for Class::DBI::Plugin::SerializeColumns, created by h2xs. It looks like the
author of the extension was negligent enough to leave the stub
unedited.

Blah blah blah.

=head2 EXPORT

None by default.



=head1 SEE ALSO

Mention other useful documentation such as the documentation of
related modules or operating system documentation (such as man pages
in UNIX), or any relevant external documentation such as RFCs or
standards.

If you have a mailing list set up for your module, mention it here.

If you have a web site set up for your module, mention it here.

=head1 AUTHOR

Masahiro Nagano, E<lt>kazeburo@gmail.comE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2005 by Masahiro Nagano

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.5 or,
at your option, any later version of Perl 5 you may have available.


=cut

