
use strict;
use Test::More;

BEGIN {
	eval "use DBD::SQLite";
	plan $@ ? (skip_all => 'needs DBD::SQLite for testing') : (tests => 7);
}

package My::Film;

use base 'Class::DBI';
use Class::DBI::Plugin::SerializeColumns;
use File::Temp qw/tempfile/;

my (undef, $DB) = tempfile();
my @DSN = ("dbi:SQLite:dbname=$DB", '', '', { AutoCommit => 1 });

END { unlink $DB if -e $DB }

__PACKAGE__->set_db(Main => @DSN);
__PACKAGE__->table('Movies');
__PACKAGE__->columns(All => qw/id title serialized/);
__PACKAGE__->serialize_columns(serialized=>qw/year category/);

sub CONSTRUCT {
	shift->db_Main->do(
		qq{
     CREATE TABLE Movies (
        id     INTEGER PRIMARY KEY,
        title  VARCHAR(255),
        serialized   blob
     )
	}
	);
}

package main;

My::Film->CONSTRUCT;

my %films = (
	Veronique =>{id=>1,year=>1991,category=>'V'},
	Red       => {id=>2,year=>1994,category=>'R'},
	White     => {id=>3,year=>1994,category=>'W'},
	Blue      => {id=>4,year=>1993,category=>'B'},
	Dekalog   => {id=>5,year=>1988,category=>'D'},
	Hospital  => {id=>6,year=>1976,category=>'H'},
	Heaven    => {id=>7,year=>2002,category=>'H'},
);

while (my ($title, $ref) = each %films) {
	My::Film->create({id=>$ref->{id}, title => $title, year => $ref->{year} , category => $ref->{category} });
}

{
	my @films = My::Film->retrieve_all;
	is @films, 7, "Got 7 films";
}


{
	my $film = My::Film->retrieve(1);
	is $film->title, 'Veronique', "Veronique title";
	is $film->year, $films{Veronique}{year}, "Veronique year";
	is $film->category, $films{Veronique}{category}, "Veronique category";
}


{
	my $film = My::Film->retrieve(2);
	$film->title('RedRed');
	$film->category('RR');
	$film->update;
}

{
	my $film = My::Film->retrieve(2);
	is $film->title, 'RedRed', "update Red title";
	is $film->year, 1994, "update Red year";
	is $film->category, "RR", "update Red category";
}

